/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.system.user.service;


import org.springblade.core.mp.base.BaseService;
import org.springblade.system.user.entity.User;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 用户查询服务类
 *
 * @author Chill
 */
public interface IUserSearchService extends BaseService<User> {

	/**
	 * 根据用户ID查询用户列表
	 *
	 * @param userId 用户ID
	 * @return 用户列表
	 */
	List<User> listByUser(List<Long> userId);

    /**
	 * 根据部门ID查询用户列表
	 *
	 * @param deptId 部门ID
	 * @return 用户列表
	 */
	List<User> listByDept(List<Long> deptId);

    /**
	 * 根据岗位ID查询用户列表
	 *
	 * @param postId 岗位ID
	 * @return 用户列表
	 */
	List<User> listByPost(List<Long> postId);

    /**
	 * 根据角色ID查询用户列表
	 *
	 * @param roleId 角色ID
	 * @return 用户列表
	 */
	List<User> listByRole(List<Long> roleId);

	/**
	 * 根据用户id查询用户名称
	 *
	 * @param idList 用户id列表
	 * @return MAP<用户id, 用户名称>
	 */
	Map<Long, String> mapByUser(Collection<Long> idList);

	/**
	 * 通过用户id获取用户名称以及岗位名称
	 *
	 * @param userIdList 用户id
	 * @return MAP<用户id, 用户名称和岗位Map>
	 */
	Map<Long, Map<String, Object>> getUserNameAndPostByUserId(List<Long> userIdList);

	/**
	 * 通过用户工号获取用户id
	 *
	 * @param jobNoList 用户工号
	 * @return MAP<用户工号, 用户id>
	 */
	Map<String, Long> getUserIdByJobNo(List<String> jobNoList);

}
