package com.fjhx.utils;


import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

/**
 * hmac签名
 *
 * @Author:caozj
 * @DATE:2022/7/27 19:51
 */
public class HmacUtil {
    //加密算法
    public static final String HMAC_SHA256 = "HmacSHA256";


    /**
     * HmacSHA256加密
     *
     * @param input 需要加密得参数
     * @param key   密钥
     * @return 加密后的值
     **/
    public static String encrypt(String input, String key) {
        String cipher = "";
        try {
            byte[] data = key.getBytes(StandardCharsets.UTF_8);
            //根据给定的字节数组构造一个密钥，第二个参数指定一个密钥的算法名称，生成HmacSHA1专属密钥
            SecretKey secretKey = new SecretKeySpec(data, HMAC_SHA256);
            //生成一个指定Mac算法的Mac对象
            Mac mac = Mac.getInstance(HMAC_SHA256);
            //用给定密钥初始化Mac对象
            mac.init(secretKey);
            byte[] text = input.getBytes(StandardCharsets.UTF_8);
            byte[] encryptByte = mac.doFinal(text);
            cipher = bytesToHexStr(encryptByte);
        } catch (NoSuchAlgorithmException | InvalidKeyException e) {
            e.printStackTrace();
        }
        return cipher;
    }


    /**
     * byte数组转16进制字符串
     *
     * @param bytes byte数组
     * @return hex字符串
     */
    public static String bytesToHexStr(byte[] bytes) {
        StringBuilder hexStr = new StringBuilder();
        for (byte b : bytes) {
            String hex = Integer.toHexString(b & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            hexStr.append(hex);
        }
        return hexStr.toString();
    }

}
