package com.fjhx.enums.flow;

import lombok.Getter;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 流程类型枚举
 */
@Getter
public enum FlowCodeEnum {

    PURCHASE(1, "PURCHASE"), //采购审批

    ;

    private final int key;

    private final String value;

    private static final Map<Integer, String> map = new LinkedHashMap<>();

    private static final HashMap<Integer, FlowCodeEnum> classMap = new HashMap<>();

    FlowCodeEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }

    static {
        for (FlowCodeEnum value : FlowCodeEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据type获取枚举
     */
    public static FlowCodeEnum get(Integer type) {
        return classMap.get(type);
    }

    /**
     * 获取枚举map
     */
    public static Map<Integer, String> getMap() {
        if (Func.isNotEmpty(map)) {
            return map;
        }
        for (FlowCodeEnum ms : values()) {
            map.put(ms.key, ms.value);
        }
        return map;
    }

    /**
     * 通过key获取名称
     */
    public static String getNameByKey(Integer key) {
        if (key == null || key < 0) {
            return StringPool.EMPTY;
        }
        Map<Integer, String> map = getMap();
        return map.getOrDefault(key, StringPool.EMPTY);
    }

}