package com.fjhx.enums.material;

import lombok.Getter;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 物料类型枚举
 */
@Getter
public enum MaterialTypeEnum {
    MATERIAL_TYPE_1(1, "原料"),
    MATERIAL_TYPE_2(2, "辅料"),
    MATERIAL_TYPE_3(3, "配件"),
    MATERIAL_TYPE_99(99, "其他"),
    ;

    private final int key;

    private final String value;

    private static final Map<Integer, String> map = new LinkedHashMap<>();

    private static final HashMap<Integer, MaterialTypeEnum> classMap = new HashMap<>();

    MaterialTypeEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }


    static {
        for (MaterialTypeEnum value : MaterialTypeEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据type获取枚举
     */
    public static MaterialTypeEnum get(Integer type) {
        return classMap.get(type);
    }

    /**
     * 获取枚举map
     */
    public static Map<Integer, String> getMap() {
        if (Func.isNotEmpty(map)) {
            return map;
        }
        for (MaterialTypeEnum ms : values()) {
            map.put(ms.key, ms.value);
        }
        return map;
    }

    /**
     * 通过key获取名称
     */
    public static String getNameByKey(Integer key) {
        if (key == null || key < 0) {
            return StringPool.EMPTY;
        }
        Map<Integer, String> map = getMap();
        return map.getOrDefault(key, StringPool.EMPTY);
    }

}
