package com.fjhx.enums.purchase;

import lombok.Getter;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 采购状态枚举
 */
@Getter
public enum PurchaseStatusEnum {
    STATUS_10(10, "审批中"),
    STATUS_20(20, "驳回"),
    STATUS_30(30, "未到货"),
    STATUS_40(40, "部分到货"),
    STATUS_50(50, "已到货"),
    STATUS_99(99, "已取消"),
    ;

    private final int key;

    private final String value;

    private static final Map<Integer, String> map = new LinkedHashMap<>();

    private static final HashMap<Integer, PurchaseStatusEnum> classMap = new HashMap<>();

    PurchaseStatusEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }

    static {
        for (PurchaseStatusEnum value : PurchaseStatusEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据type获取枚举
     */
    public static PurchaseStatusEnum get(Integer type) {
        return classMap.get(type);
    }

    /**
     * 获取枚举map
     *
     * @return
     */
    public static Map<Integer, String> getMap() {
        if (Func.isNotEmpty(map)) {
            return map;
        }
        for (PurchaseStatusEnum ms : values()) {
            map.put(ms.key, ms.value);
        }
        return map;
    }

    /**
     * 通过key获取名称
     */
    public static String getNameByKey(Integer key) {
        if (key == null || key < 0) {
            return StringPool.EMPTY;
        }
        Map<Integer, String> map = getMap();
        return map.getOrDefault(key, StringPool.EMPTY);
    }

}
