package com.fjhx.enums.stock;

import cn.hutool.core.util.ObjectUtil;
import lombok.Getter;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.Map;

/**
 * @author 入库类型枚举
 */
@Getter
public enum InTypeEnum {
    // business_id = logistics_details 表 id
    PURCHASE(1, "采购入库"),

    // business_id = 自定义id
    MANUAL(2, "手动入库"),

    // business_id = stock_transfer 表 id
    TRANSFER(7, "调仓入库"),

    // business_id = stock_combination 表 id
    COMBINATION(9, "组合入库"),

    ;

    private final int key;

    private final String value;

    InTypeEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }

    private static final Map<Integer, InTypeEnum> CLASS_MAP = new HashMap<>();

    static {
        for (InTypeEnum value : InTypeEnum.values()) {
            CLASS_MAP.put(value.getKey(), value);
        }
    }

    /**
     * 根据key获取枚举
     */
    public static InTypeEnum getEnumByKey(Integer type) {
        return CLASS_MAP.get(type);
    }

    /**
     * 通过key获取名称
     */
    public static String getValueByKey(Integer key) {
        InTypeEnum inTypeEnum = CLASS_MAP.get(key);

        if (ObjectUtil.isEmpty(inTypeEnum)) {
            return StringPool.EMPTY;
        }

        return inTypeEnum.getValue();
    }

}
