/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package com.fjhx.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.base.Condition;
import com.fjhx.params.TenantVo;
import com.fjhx.service.ITenantService;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.tenant.annotation.NonDS;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.constant.RoleConstant;
import org.springblade.system.entity.Tenant;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * 控制器
 *
 * @author Chill
 */
@NonDS
@RestController
@AllArgsConstructor
@RequestMapping("/tenant")
public class TenantController extends BladeController {

    private final ITenantService tenantService;

    /**
     * 新增或修改
     */
    @PostMapping("/submit")
    @PreAuth(RoleConstant.HAS_ROLE_ADMINISTRATOR)
    public R submit(@Valid @RequestBody Tenant tenant) {
        tenantService.submitTenant(tenant);
        return R.success();
    }

    /**
     * 分页
     */
    @PostMapping("/page")
    @PreAuth(RoleConstant.HAS_ROLE_ADMINISTRATOR)
    public R page(@RequestBody Condition condition) {
        Page<Map<String, Object>> page = tenantService.getPage(condition);
        return R.success(page);
    }

    /**
     * 第三方创建租户
     */
    @PostMapping("/thirdPartyCreate")
    public R thirdPartyCreate(@RequestBody Condition condition) {
        TenantVo token = tenantService.thirdPartyCreate(condition.getStr("token"));
        return R.success(token);
    }

}
