package com.fjhx.supplier.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fjhx.entity.supplier.Supplier;
import org.apache.ibatis.annotations.Param;
import org.springblade.core.tenant.annotation.TenantIgnore;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 供应商 Mapper 接口
 * </p>
 *
 * @author ${author}
 * @since 2022-07-12
 */
public interface SupplierMapper extends BaseMapper<Supplier> {

    /**
     * 全部列表
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    List<Supplier> getSumList(Map<String, Object> condition);

    /**
     * 全部列表-总条数
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    Integer getSumListCount(Map<String, Object> condition);

    /**
     * 列表
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    List<Supplier> getList(Map<String, Object> condition);

    /**
     * 列表-总条数
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    Integer getListCount(Map<String, Object> condition);

    /**
     * 供应商采购列表
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    List<Supplier> getPurList(Map<String, Object> condition);

    /**
     * 供应商采购列表-总条数
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    Integer getPurListCount(Map<String, Object> condition);

    /**
     * 物料分类分组查询
     *
     * @return
     */
    @TenantIgnore
    List<Supplier> getGroupByCategory(Map<String, Object> condition);

    /**
     * 供应商详情
     *
     * @return
     */
    @TenantIgnore
    Supplier getSuById(String id);

    /**
     * 获取采购物料款数
     *
     * @param id
     * @return
     */
    @TenantIgnore
    Integer getMaterialTypeCount(String id);

    /**
     * 获取供应商退货次数
     *
     * @param id
     * @return
     */
    @TenantIgnore
    Integer getBackCount(String id);

    /**
     * 供应商采购物料分析 列表
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    List<Supplier> getSuPurMaterialAnalyse(Map<String, Object> condition);

    /**
     * 供应商采购物料分析 条数
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    Integer getSuPurMaterialAnalyseCount(Map<String, Object> condition);

    /**
     * 月度退货记录分析
     *
     * @param id
     * @return
     */
    @TenantIgnore
    List<Supplier> getBackSumMoneyAndCount(@Param("id") String id, @Param("date") String date);

    /**
     * 退货记录总数和总金额
     *
     * @param id
     * @return
     */
    @TenantIgnore
    Map<String, Object> getAllSumMoneyAndCount(@Param("id") String id, @Param("date") String date);

    /**
     * 月度采购分析
     *
     * @param id
     * @param date
     * @return
     */
    @TenantIgnore
    List<Supplier> getMonthSumPurMoney(@Param("id") String id, @Param("date") String date);


    /**
     * 下拉列表
     *
     * @param condition
     * @return
     */
    @TenantIgnore
    List<Supplier> selectLists(Map<String, Object> condition);

    @TenantIgnore
    List<Map<String, Object>> getSafetyWarnDayList();

    @TenantIgnore
    List<Map<String, Object>> getSafetyWarnDayBySupplier(@Param("supplierId") String supplierId);

    @TenantIgnore
    BigDecimal paymentGetBackAmount(@Param("purchaseBillNo") String purchaseBillNo);

}
